//
//  KQ2_MainViewController.m
//  KingsQueensNav12
//
//  Created by Terry Payne on 31/10/2012.
//  Copyright (c) 2012 Terry Payne. All rights reserved.
//

#import "KQ2_MainViewController.h"
#import "KQ2_CustomCell.h"
#import "KQ2_DataSourceMacros.h"
#import "KQ2_DetailViewController.h"

static NSString *myJSONDataSrc = @"http://www.csc.liv.ac.uk/people/trp/Teaching_Resources/COMP327/Monarchs6.json";

#define BIRTHLABEL_TAG 1
#define DEATHLABEL_TAG 2
#define BIRTHIMAGE_TAG 3
#define DEATHIMAGE_TAG 4

@interface KQ2_MainViewController ()

@property (strong, nonatomic) NSDictionary *myKingsQueensDataModel;
@property (strong, nonatomic) NSArray *monarchHouseKeys;

@end

@implementation KQ2_MainViewController

// The following are only needed if running the code on XCode 4.4 or earlier
// @synthesize myKingsQueensDataModel = _myKingsQueensDataModel;  // only for early versions of Xcode pre 4.5
// @synthesize monarchHouseKeys = _monarchHouseKeys;  // only for early versions of Xcode pre 4.5


// ==================================================================
// Lazy instantiation
// ==================================================================

- (NSDictionary *) myKingsQueensDataModel {
    if (_myKingsQueensDataModel == nil) {
        // Calling the getter method for the first time, so we need to load the data.
        NSLog(@"Calling accessor method myKingsQueensDataModel for the first time...");
        
        // Generate the URL request for the JSON data
        NSURL *url = [NSURL URLWithString:myJSONDataSrc];
        
        // Get the contents of the URL
        NSData *data = [NSData dataWithContentsOfURL:url];
        NSError *error;
        // Note that we are not calling the setter method here... as this would be recursive!!!
        _myKingsQueensDataModel = [NSJSONSerialization JSONObjectWithData:data options:kNilOptions error:&error];
    }
    return _myKingsQueensDataModel;
}

- (NSArray *) monarchHouseKeys {
    if (_monarchHouseKeys == nil) {
        // Note, we create a mutable array to temporarily store the keys, then
        // copy this into the immutable iVar array _monarchHouseKeys
        NSString *keyStr;
        NSMutableArray *tmpArr = [[NSMutableArray alloc] init];
        for (keyStr in [self myKingsQueensDataModel]) {
            [tmpArr addObject:keyStr];
        }
        _monarchHouseKeys = [NSArray arrayWithArray:tmpArr];
    }
    return _monarchHouseKeys;
}

// ==================================================================
#pragma mark - Life Cycle

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    UINib *monarchCellNib = [UINib nibWithNibName:@"KQ2_CustomCell" bundle:nil];
    [[self tableView] registerNib:monarchCellNib forCellReuseIdentifier:CUSTOM_CELL_REUSE_IDENTIFIER];
    
    // Calculate the height of the cells
    // As the nib has already been loaded through the creation of monarchCellNib
    // we can simply pull out the cell from that and then calculate its bounds.
    // The following code fragment "does the right thing" by checking that the only
    // object we consider is of type KQ2_CustomCell
    
    NSArray *nibArray = [monarchCellNib instantiateWithOwner:nil options:nil];
    KQ2_CustomCell *cell;
    for (id currentObject in nibArray) {
        if ([currentObject isKindOfClass:[KQ2_CustomCell class]]) {
            cell = (KQ2_CustomCell *) currentObject;
            break;
        }
    }

    if (cell) {
        CGRect cellRect = [cell bounds];
        [[self tableView] setRowHeight:cellRect.size.height];
    }

    [self setTitle:@"UK Monarchs"];
    
    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return [[self monarchHouseKeys] count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    NSString *key = [[self monarchHouseKeys] objectAtIndex:section];
    NSDictionary *houseObject = [[self myKingsQueensDataModel] objectForKey:key];
    NSArray *monarchsArray = [houseObject objectForKey:MONARCHS];
    return [monarchsArray count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    KQ2_CustomCell *cell = [tableView dequeueReusableCellWithIdentifier:CUSTOM_CELL_REUSE_IDENTIFIER forIndexPath:indexPath];
    
    // Configure the cell...
    
    // Unpack the data structure
    NSString *key = [[self monarchHouseKeys] objectAtIndex:[indexPath section]];
    NSDictionary *monarchDict = [[self myKingsQueensDataModel] objectForKey:key];
    NSArray *monarchs = [monarchDict objectForKey:MONARCHS];
    
    // Set the main label text details
    [[cell nameLabel] setText:[[monarchs objectAtIndex:[indexPath row]] objectForKey:NAME]];
    [[cell reignedLabel] setText:[[monarchs objectAtIndex:[indexPath row]] objectForKey:REIGNED]];
    
    // Display thumb images
    NSURL *url = [NSURL URLWithString:[[monarchs objectAtIndex:[indexPath row]] objectForKey:@"thumb"]];
    UIImage *thumbImage = [UIImage imageWithData:[NSData dataWithContentsOfURL:url]];
    [[cell thumbImageView] setImage:thumbImage];
    
    // set the details for the birth and death lables/images
    [[cell birthLabel] setText:[[monarchs objectAtIndex:[indexPath row]] objectForKey:BORN]];
    [[cell deathLabel ] setText:[[monarchs objectAtIndex:[indexPath row]] objectForKey:DIED]];
    
    return cell;
}


//- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
//{
//    static NSString *CellIdentifier = @"MonarchCell";
//    
//    UILabel *birthLabel, *deathLabel;
//    UIImageView *birthImage, *deathImage;
//    
//    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
//    if (cell == nil) {
//        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
//        
//        // ===========================
//        birthLabel = [[UILabel alloc]
//                       initWithFrame:CGRectMake(220.0, 5.0, 80.0, 15.0)];
//        [birthLabel setFont:[UIFont systemFontOfSize:11.0]];
//        [birthLabel setTag:BIRTHLABEL_TAG];
//        [[cell contentView] addSubview:birthLabel];
//        
//        deathLabel = [[UILabel alloc]
//                       initWithFrame:CGRectMake(220.0, 25.0, 80.0, 15.0)];
//        [deathLabel setFont:[UIFont systemFontOfSize:11.0]];
//        [deathLabel setTag:DEATHLABEL_TAG];
//        [[cell contentView] addSubview:deathLabel];
//        
//        birthImage = [[UIImageView alloc]
//                       initWithFrame:CGRectMake(200.0, 5.0, 13.0, 19.0)];
//        [birthImage setTag:BIRTHIMAGE_TAG];
//        [[cell contentView] addSubview:birthImage];
//        
//        deathImage = [[UIImageView alloc]
//                       initWithFrame:CGRectMake(200.0, 25.0, 14.0, 13.0)];
//        [deathImage setTag:DEATHIMAGE_TAG];
//        [[cell contentView] addSubview:deathImage];
//    } else {
//        birthLabel = (UILabel *)[[cell contentView] viewWithTag:BIRTHLABEL_TAG];
//        deathLabel = (UILabel *)[[cell contentView] viewWithTag:DEATHLABEL_TAG];
//        birthImage = (UIImageView *)[[cell contentView] viewWithTag:BIRTHIMAGE_TAG];
//        deathImage = (UIImageView *)[[cell contentView] viewWithTag:DEATHIMAGE_TAG];
//    }
//        // ===========================
//    
//    // Configure the cell...
//    
//    // Unpack the data structure
//    NSString *key = [[self monarchHouseKeys] objectAtIndex:[indexPath section]];
//    NSDictionary *monarchDict = [[self myKingsQueensDataModel] objectForKey:key];
//    NSArray *monarchs = [monarchDict objectForKey:MONARCHS];
//        
//    // Set the main label text details
//    [[cell textLabel] setText:[[monarchs objectAtIndex:[indexPath row]] objectForKey:NAME]];
//    [[cell detailTextLabel] setText:[[monarchs objectAtIndex:[indexPath row]] objectForKey:REIGNED]];
//    
//    
//    
//    // Display thumb images
//    NSURL *url = [NSURL URLWithString:[[monarchs objectAtIndex:[indexPath row]] objectForKey:@"thumb"]];
//    UIImage *thumbImage = [UIImage imageWithData:[NSData dataWithContentsOfURL:url]];
//    [[cell imageView] setImage:thumbImage];
//    
//    // set the details for the birth and death lables/images
//    [birthLabel setText:[[monarchs objectAtIndex:[indexPath row]] objectForKey:BORN]];
//    [deathLabel setText:[[monarchs objectAtIndex:[indexPath row]] objectForKey:DIED]];
//
//    // ============================
//    
//    NSString *birthImagePath = [[NSBundle mainBundle] pathForResource:@"76-baby" ofType:@"png"];
//    UIImage *theBirthImage = [UIImage imageWithContentsOfFile:birthImagePath];
//    [birthImage setImage:theBirthImage];
//    
//    NSString *deathImagePath = [[NSBundle mainBundle] pathForResource:@"21-skull" ofType:@"png"];
//    UIImage *theDeathImage = [UIImage imageWithContentsOfFile:deathImagePath];
//    [deathImage setImage:theDeathImage];
//    // ============================
//    return cell;
//}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    NSString *key = [[self monarchHouseKeys] objectAtIndex:section];
    NSDictionary *houseObject = [[self myKingsQueensDataModel] objectForKey:key];
    return [houseObject objectForKey:TITLE];
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section
{
    NSString *key = [[self monarchHouseKeys] objectAtIndex:section];
    NSDictionary *houseObject = [[self myKingsQueensDataModel] objectForKey:key];
    return [houseObject objectForKey:EPOCH];
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller
    KQ2_DetailViewController *detailViewController = [[KQ2_DetailViewController alloc]
                                                      initWithNibName:@"KQ2_DetailViewController" bundle:nil];
    // ...
    
    // Unpack the data structure
    NSString *key = [[self monarchHouseKeys] objectAtIndex:[indexPath section]];
    NSDictionary *monarchDict = [[self myKingsQueensDataModel] objectForKey:key];
    NSArray *monarchs = [monarchDict objectForKey:MONARCHS];
    [detailViewController setMonarchDetails:[monarchs objectAtIndex:[indexPath row]]];

    // Create a temporary dictionary to pass through the two house elements to the view controller
    // This avoids having to pass through potentially several objects.
    NSDictionary *tmpHouseDetails = @{HOUSETITLE:[monarchDict objectForKey:HOUSETITLE],
                                      HOUSEEPOCH:[monarchDict objectForKey:HOUSEEPOCH]};
    [detailViewController setHouseDetails:tmpHouseDetails];    
    
    // Pass the selected object to the new view controller.
    [self.navigationController pushViewController:detailViewController animated:YES];
    
    
}

@end
